@@ -40,7 +40,6 @@ class CommandDispatcher(MixinBase):
     def __init__(self: "Anjani", **kwargs: Any) -> None:
         # Initialize command map
         self.commands = {}
-        self.limiter = util.cache_limiter.CacheLimiter(ttl=10, max_value=3)
 
         # Propagate initialization to other mixins
         super().__init__(**kwargs)
@@ -133,33 +132,41 @@ class CommandDispatcher(MixinBase):
                 return False  # ignore channel broadcasts
 
             if message.text is not None and message.text.startswith("/"):
-                parts = message.text.split()
-                parts[0] = parts[0][1:]
-
-                # Check if bot command contains a valid username
-                # eg: /ping@dAnjani_bot will return True
-                # If current bot instance is dAnjani_bot else False
-                if self.user.username and self.user.username in parts[0]:
-                    # Remove username from command
-                    parts[0] = parts[0].replace(f"@{self.user.username}", "")
-
-                # Filter if command is not in commands
-                try:
-                    cmd = self.commands[parts[0]]
-                except KeyError:
+                user = message.from_user or message.sender_chat
+                if await self._limiter.exceeded(user.id):
                     return False
 
-                # Check additional built-in filters
-                if cmd.filters:
-                    if inspect.iscoroutinefunction(cmd.filters.__call__):
-                        if not await cmd.filters(client, message):
-                            return False
-                    else:
-                        if not await util.run_sync(cmd.filters, client, message):
-                            return False
+                try:
+                    parts = message.text.split()
+                    parts[0] = parts[0][1:]
 
-                message.command = parts
-                return True
+                    # Check if bot command contains a valid username
+                    # eg: /ping@dAnjani_bot will return True
+                    # If current bot instance is dAnjani_bot else False
+                    if self.user.username and self.user.username in parts[0]:
+                        # Remove username from command
+                        parts[0] = parts[0].replace(f"@{self.user.username}", "")
+
+                    # Filter if command is not in commands
+                    try:
+                        cmd = self.commands[parts[0]]
+                    except KeyError:
+                        return False
+
+                    # Check additional built-in filters
+                    if cmd.filters:
+                        if inspect.iscoroutinefunction(cmd.filters.__call__):
+                            if not await cmd.filters(client, message):
+                                return False
+                        else:
+                            if not await util.run_sync(cmd.filters, client, message):
+                                return False
+
+                    message.command = parts
+                    return True
+                finally:
+                    # Increment user limit
+                    await self._limiter.increment(user.id)
 
             return False
 
@@ -168,11 +175,6 @@ class CommandDispatcher(MixinBase):
     async def on_command(
         self: "Anjani", client: Client, message: Message  # skipcq: PYL-W0613
     ) -> None:
-        # Limiter checking here
-        user_id = message.from_user.id
-        if not await self.limiter.check_rate_limit(user_id):
-            return
-
         # cmd never raises KeyError because we checked on command_predicate
         cmd = self.commands[message.command[0]]
         try:
@@ -220,12 +222,13 @@ class CommandDispatcher(MixinBase):
                     "    • Input   -> %s\n",
                     cmd.name,
                     chat.title if chat else None,
-                    chat.id if chat else None,
+                    chat.id if chat else -1,
                     user.first_name if user else None,
-                    user.id if user else None,
+                    user.id if user else -1,
                     ctx.input,
                     exc_info=constructor_invoke,
                 )
+                await self.dispatch_alert(f"command `/{message.command[0]}`", constructor_invoke)
 
             await self.dispatch_event("command", ctx, cmd)
         except Exception as e:  # skipcq: PYL-W0703
@@ -248,9 +251,9 @@ class CommandDispatcher(MixinBase):
                 message.command,
                 exc_info=constructor_handler,
             )
+            await self.dispatch_alert(
+                f"command `/{message.command[0]}` handler", constructor_handler
+            )
         finally:
-            # Increment user count to cached
-            await self.limiter.increment_rate_limit(user_id)
-
             # Continue processing handler of on_message
             raise ContinuePropagation
