@@ -42,7 +42,7 @@ from pyrogram.errors import (
     PeerIdInvalid,
     UserNotParticipant,
 )
-from pyrogram.types import ChatPermissions, Message
+from pyrogram.types import Chat, ChatPermissions, Message
 
 from anjani import command, filters, plugin, util
 
@@ -144,7 +144,7 @@ class Lockings(plugin.Plugin):
             if message.sender_chat.id == chat.id:  # anon admin
                 return
 
-            current_chat: Any = await self.bot.get_chat(chat.id)
+            current_chat: Chat = await self.bot.get_chat(chat.id)
             if current_chat.linked_chat and message.sender_chat.id == current_chat.linked_chat.id:
                 # Linked channel group
                 return
@@ -240,7 +240,6 @@ class Lockings(plugin.Plugin):
                     "messages": {"can_send_messages": self.get_mode(mode)},
                     "media": {"can_send_media_messages": self.get_mode(mode)},
                     "polls": {"can_send_polls": self.get_mode(mode)},
-                    "other": {"can_send_other_messages": self.get_mode(mode)},
                     "previews": {"can_add_web_page_previews": self.get_mode(mode)},
                     "info": {"can_change_info": self.get_mode(mode)},
                     "invite": {"can_invite_users": self.get_mode(mode)},
@@ -313,10 +312,13 @@ class Lockings(plugin.Plugin):
             ):
                 return await ctx.get_text("lockings-type-locked", lock_type=lock_type)
 
-            await self.bot.client.set_chat_permissions(
-                chat_id=chat.id,
-                permissions=self.unpack_permissions(permissions, "lock", lock_type),
-            )
+            try:
+                await self.bot.client.set_chat_permissions(
+                    chat_id=chat.id,
+                    permissions=self.unpack_permissions(permissions, "lock", lock_type),
+                )
+            except ChatAdminRequired as e:
+                return await ctx.get_text("lockings-admin-required", message=e.MESSAGE.split()[-1].strip(")").split(".")[-1].strip('"') if e.MESSAGE else "")
         elif lock_type in LOCK_TYPES:
             locked = await self.get_chat_restrictions(chat.id)
             if lock_type in locked:
