@@ -16,15 +16,39 @@
 
 import asyncio
 from html import escape
-from typing import Any, ClassVar, MutableMapping, Optional, Tuple
+from typing import (
+    Any,
+    Callable,
+    ClassVar,
+    Coroutine,
+    List,
+    MutableMapping,
+    Optional,
+    Tuple,
+    Union,
+)
 
 from pyrogram.client import Client
 from pyrogram.enums.parse_mode import ParseMode
-from pyrogram.errors import ChannelPrivate, ChatWriteForbidden, MessageDeleteForbidden
+from pyrogram.errors import (
+    ChannelPrivate,
+    ChatWriteForbidden,
+    MediaEmpty,
+    MessageDeleteForbidden,
+    MessageEmpty,
+)
 from pyrogram.types import Chat, Message, User
 from pyrogram.types.messages_and_media.message import Str
 
 from anjani import command, filters, plugin, util
+from anjani.util.tg import (
+    Button,
+    Types,
+    build_button,
+    get_message_info,
+    parse_button,
+    revert_button,
+)
 
 
 class Greeting(plugin.Plugin):
@@ -32,9 +56,25 @@ class Greeting(plugin.Plugin):
     helpable: ClassVar[bool] = True
 
     db: util.db.AsyncCollection
+    chat_db: util.db.AsyncCollection
+    SEND: MutableMapping[int, Callable[..., Coroutine[Any, Any, Optional[Message]]]]
 
     async def on_load(self) -> None:
         self.db = self.bot.db.get_collection("WELCOME")
+        self.chat_db = self.bot.db.get_collection("CHATS")
+
+        self.SEND = {
+            Types.TEXT.value: self.bot.client.send_message,
+            Types.BUTTON_TEXT.value: self.bot.client.send_message,
+            Types.DOCUMENT.value: self.bot.client.send_document,
+            Types.PHOTO.value: self.bot.client.send_photo,
+            Types.VIDEO.value: self.bot.client.send_video,
+            Types.STICKER.value: self.bot.client.send_sticker,
+            Types.AUDIO.value: self.bot.client.send_audio,
+            Types.VOICE.value: self.bot.client.send_voice,
+            Types.VIDEO_NOTE.value: self.bot.client.send_video_note,
+            Types.ANIMATION.value: self.bot.client.send_animation,
+        }
 
     async def on_chat_action(self, message: Message) -> None:
         chat = message.chat
@@ -50,13 +90,20 @@ class Greeting(plugin.Plugin):
                 pass
             reply_to = 0
 
+        thread_id = await self.get_action_topic(chat)
+        if message.chat.is_forum and not thread_id:
+            self.log.debug(f"Chat {message.chat.id} is forum but no action topic set!")
+            # continue try to send on default (general) topic
+
         if message.new_chat_members:
-            return await self._member_join(message, reply_to)
+            return await self._member_join(message, reply_to, thread_id)
 
         if message.left_chat_member:
-            return await self._member_leave(message, reply_to)
+            return await self._member_leave(message, reply_to, thread_id)
 
-    async def _member_leave(self, message: Message, reply_to: int) -> None:
+    async def _member_leave(
+        self, message: Message, reply_to: int, thread_id: Optional[int]
+    ) -> None:
         chat = message.chat
         if not await self.is_goodbye(chat.id):
             return
@@ -71,7 +118,8 @@ class Greeting(plugin.Plugin):
             msg = await self.bot.client.send_message(
                 chat.id,
                 formatted_text,
-                reply_to_message_id=reply_to,
+                reply_to_message_id=reply_to if not thread_id else None,  # type: ignore
+                message_thread_id=thread_id,  # type: ignore
             )
         except ChatWriteForbidden:
             return
@@ -83,13 +131,14 @@ class Greeting(plugin.Plugin):
             except MessageDeleteForbidden:
                 pass
 
-    async def _member_join(self, message: Message, reply_to: int) -> None:
+    async def _member_join(self, message: Message, reply_to: int, thread_id: Optional[int]) -> None:
         chat = message.chat
         if not await self.is_welcome(chat.id):
             return
 
         new_members = message.new_chat_members
-        for new_member in new_members:
+        is_bulk_welcome = len(new_members) > 1
+        for idx, new_member in enumerate(new_members):
             try:
                 if new_member.id == self.bot.uid:
                     await self.bot.client.send_message(
@@ -98,7 +147,8 @@ class Greeting(plugin.Plugin):
                         reply_to_message_id=reply_to,
                     )
                 else:
-                    text, button = await self.welc_message(chat.id)
+                    text, button, msg_type, file_id = await self.welc_message(chat.id)
+                    msg_type = Types(msg_type) if msg_type else Types.TEXT
                     if not text:
                         string = await self.text(chat.id, "default-welcome", noformat=True)
                     else:
@@ -109,26 +159,51 @@ class Greeting(plugin.Plugin):
                     )
 
                     if button:
-                        button = util.tg.build_button(button)
-                        msg = await self.bot.client.send_message(
-                            chat.id,
-                            formatted_text,
-                            reply_to_message_id=reply_to,
-                            reply_markup=button,
-                        )
+                        button = build_button(button)
                     else:
-                        msg = await self.bot.client.send_message(
-                            chat.id,
-                            formatted_text,
-                            reply_to_message_id=reply_to,
+                        button = None
+                    msg = None
+                    try:
+                        if msg_type in {Types.TEXT, Types.BUTTON_TEXT}:
+                            msg = await self.SEND[msg_type](
+                                message.chat.id,
+                                formatted_text,
+                                message_thread_id=thread_id,
+                                reply_to_message_id=reply_to,
+                                reply_markup=button,
+                                disable_web_page_preview=True,
+                            )
+                        elif msg_type in {Types.STICKER, Types.ANIMATION}:
+                            msg = await self.SEND[msg_type](
+                                message.chat.id,
+                                file_id,
+                                message_thread_id=thread_id,
+                                reply_to_message_id=reply_to,
+                            )
+                        else:
+                            msg = await self.SEND[msg_type](
+                                message.chat.id,
+                                file_id,
+                                caption=formatted_text,
+                                message_thread_id=thread_id,
+                                reply_to_message_id=reply_to,
+                                reply_markup=button,
+                            )
+                    except MediaEmpty:
+                        await self.bot.client.send_message(
+                            message.chat.id,
+                            await self.text(message.chat.id, "welcome-message-expired"),
                         )
+                    except MessageEmpty:
+                        self.log.warning("Welcome message empty on %s.", message.chat.id)
 
-                    previous = await self.previous_welcome(chat.id, msg.id)
-                    if previous:
-                        try:
-                            await self.bot.client.delete_messages(chat.id, previous)
-                        except MessageDeleteForbidden:
-                            pass
+                    if msg:
+                        previous = await self.previous_welcome(chat.id, msg.id, is_bulk_welcome)
+                        if idx == 0 and previous:
+                            try:
+                                await self.bot.client.delete_messages(chat.id, previous)
+                            except MessageDeleteForbidden:
+                                pass
             except ChatWriteForbidden:
                 pass
 
@@ -171,6 +246,12 @@ class Greeting(plugin.Plugin):
             id=user.id,
         )
 
+    async def get_action_topic(self, chat: Chat) -> Optional[int]:
+        if not chat.is_forum:
+            return None
+        data = await self.chat_db.find_one({"chat_id": chat.id}, {"action_topic": True})
+        return data.get("action_topic") if data else None
+
     async def is_welcome(self, chat_id: int) -> bool:
         """Get chat welcome setting"""
         active = await self.db.find_one({"chat_id": chat_id}, {"should_welcome": 1})
@@ -183,13 +264,34 @@ class Greeting(plugin.Plugin):
 
     async def welc_message(
         self, chat_id: int
-    ) -> Tuple[Optional[str], Optional[Tuple[Tuple[str, str, bool]]]]:
+    ) -> Tuple[Optional[str], Optional[Tuple[Tuple[str, str, bool]]], Optional[int], Optional[str]]:
         """Get chat welcome string"""
-        message = await self.db.find_one({"chat_id": chat_id}, {"custom_welcome": 1, "button": 1})
+        message = await self.db.find_one({"chat_id": chat_id})
         if message:
-            return message.get("custom_welcome"), message.get("button")
-
-        return await self.text(chat_id, "default-welcome", noformat=True), None
+            # This checks data for old welcome schema
+            # TODO: deprecate old schema on v3
+            if "custom_welcome" in message:
+                text: str = message["custom_welcome"]
+                button: Optional[Button] = message.get("button")
+                message_type: Types = Types.TEXT
+                await self.db.delete_one({"chat_id": chat_id})
+                await self.set_custom_welcome(
+                    chat_id=chat_id,
+                    text=text,
+                    buttons=button,
+                    message_type=message_type,
+                    content=None,
+                )
+                self.log.info("Migrated old welcome message on %d to new schema.", chat_id)
+                return text, button, message_type, None
+            else:
+                return (
+                    message.get("text"),
+                    message.get("button"),
+                    message.get("type"),
+                    message.get("file_id"),
+                )
+        return await self.text(chat_id, "default-welcome", noformat=True), None, None, None
 
     async def left_message(self, chat_id: int) -> str:
         message = await self.db.find_one({"chat_id": chat_id}, {"custom_goodbye": 1})
@@ -209,12 +311,25 @@ class Greeting(plugin.Plugin):
 
         return False  # Defaults off
 
-    async def set_custom_welcome(self, chat_id: int, text: Str) -> None:
+    async def set_custom_welcome(
+        self,
+        chat_id: int,
+        text: str,
+        message_type: Types,
+        buttons: Optional[Button] = None,
+        content: Optional[str] = None,
+    ) -> None:
         """Set custom welcome"""
-        msg, button = util.tg.parse_button(text.markdown)
         await self.db.update_one(
             {"chat_id": chat_id},
-            {"$set": {"custom_welcome": msg, "button": button}},
+            {
+                "$set": {
+                    "text": text,
+                    "button": buttons,
+                    "file_id": content,
+                    "type": message_type,
+                }
+            },
             upsert=True,
         )
 
@@ -225,7 +340,16 @@ class Greeting(plugin.Plugin):
     async def del_custom_welcome(self, chat_id: int) -> None:
         """Delete custom welcome message"""
         await self.db.update_one(
-            {"chat_id": chat_id}, {"$unset": {"custom_welcome": "", "button": ""}}
+            {"chat_id": chat_id},
+            {
+                "$unset": {
+                    "custom_welcome": "",
+                    "text": "",
+                    "button": "",
+                    "type": "",
+                    "file_id": "",
+                }
+            },
         )
 
     async def del_custom_goodbye(self, chat_id: int) -> None:
@@ -239,10 +363,13 @@ class Greeting(plugin.Plugin):
         else:
             await self.db.update_one({"chat_id": chat_id}, {"$unset": {key: ""}}, upsert=True)
 
-    async def previous_welcome(self, chat_id: int, msg_id: int) -> Optional[int]:
+    async def previous_welcome(
+        self, chat_id: int, msg_id: int, is_bulk: bool = False
+    ) -> Union[int, List[int], None]:
         """Save latest welcome msg_id and return previous msg_id"""
+        operator = "$push" if is_bulk else "$set"
         data = await self.db.find_one_and_update(
-            {"chat_id": chat_id}, {"$set": {"prev_welc": msg_id}}, upsert=True
+            {"chat_id": chat_id}, {operator: {"prev_welc": msg_id}}, upsert=True
         )
         return data.get("prev_welc", None) if data else None
 
@@ -256,20 +383,41 @@ class Greeting(plugin.Plugin):
     async def cmd_setwelcome(self, ctx: command.Context) -> str:
         """Set chat welcome message"""
         chat = ctx.chat
+
         if ctx.input:
-            welc_text = Str(ctx.input).init(ctx.msg.entities[1:])
+            if ctx.message.media:
+                # TODO: Add support for command in media caption
+                return await self.text(chat.id, "unsupported-media-command")
+            else:
+                welc_text = (
+                    Str(ctx.message.text)
+                    .init(ctx.msg.entities)
+                    .markdown.split(ctx.invoker, 1)[1]
+                    .strip()
+                )
+                welc_text, buttons = parse_button(welc_text)
+                types = Types.TEXT
+                content = None
+                if ctx.msg.reply_to_message:
+                    _, types, content, __ = get_message_info(ctx.msg)
         elif ctx.msg.reply_to_message:
-            welc_text = ctx.msg.reply_to_message.text or ctx.msg.reply_to_message.caption
+            welc_text, types, content, buttons = get_message_info(ctx.msg)
         else:
             return await self.text(chat.id, "greetings-no-input")
 
+        if not welc_text:
+            return await self.text(chat.id, "greetings-button-only-error")
+
         try:  # Try to build a text first to check message validity
-            await self._build_text(welc_text or "", ctx.author, chat, self.bot.client)
+            await self._build_text(
+                welc_text or "", ctx.author or self.bot.user, chat, self.bot.client
+            )
         except (KeyError, ValueError) as e:
             return await self.text(chat.id, "err-msg-format-parsing", err=e)
 
         ret, _ = await asyncio.gather(
-            self.text(chat.id, "cust-welcome-set"), self.set_custom_welcome(chat.id, welc_text)
+            self.text(chat.id, "cust-welcome-set"),
+            self.set_custom_welcome(chat.id, welc_text, types, buttons, content),
         )
         return ret
 
@@ -331,7 +479,11 @@ class Greeting(plugin.Plugin):
             )
             return ret
 
-        setting, (text, button), clean_service = await asyncio.gather(
+        (
+            setting,
+            (text, button, msg_type, file_id),
+            clean_service,
+        ) = await asyncio.gather(
             self.is_welcome(chat.id), self.welc_message(chat.id), self.clean_service(chat.id)
         )
 
@@ -343,28 +495,65 @@ class Greeting(plugin.Plugin):
         if noformat:
             parse_mode = ParseMode.DISABLED
             if button:
-                text += util.tg.revert_button(button)
+                text += revert_button(button)
             button = None
         else:
             parse_mode = ParseMode.MARKDOWN
             if button:
-                button = util.tg.build_button(button)
+                button = build_button(button)
 
-        await ctx.respond(await self.text(chat.id, "view-welcome", setting, clean_service))
-        await ctx.respond(
-            text
-            if text
-            else (
-                "Empty, custom welcome message haven't set yet."
-                if not setting
-                else "Default:\n\n" + await self.text(chat.id, "default-welcome", noformat=True)
-            ),
-            mode="reply",
-            reply_markup=button,
-            parse_mode=parse_mode,
-            disable_web_page_preview=True,
-        )
-        return None
+        view_welc = await self.text(chat.id, "view-welcome", setting, clean_service)
+        if ctx.chat.is_forum and not await self.get_action_topic(ctx.chat):
+            view_welc += "\n\n" + await self.text(
+                chat.id,
+                "greetings-topic-default",
+                f"https://t.me/{self.bot.user.username}?start=help_topic",
+            )
+
+        settings_msg = await ctx.respond(view_welc)
+
+        reply_to = settings_msg.id if settings_msg else None
+        try:
+            response_text = (
+                text
+                if text
+                else (
+                    "Empty, custom welcome message haven't set yet."
+                    if not setting
+                    else "Default:\n\n" + await self.text(chat.id, "default-welcome", noformat=True)
+                )
+            )
+            msg_type = msg_type or Types.TEXT
+            if msg_type in {Types.TEXT, Types.BUTTON_TEXT}:
+                await self.SEND[msg_type](
+                    ctx.chat.id,
+                    response_text,
+                    reply_to_message_id=reply_to,
+                    reply_markup=button,
+                    parse_mode=parse_mode,
+                    disable_web_page_preview=True,
+                )
+            elif msg_type in {Types.STICKER, Types.ANIMATION}:
+                await self.SEND[msg_type](
+                    ctx.chat.id,
+                    file_id,
+                    reply_to_message_id=reply_to,
+                )
+            else:
+                await self.SEND[msg_type](
+                    ctx.chat.id,
+                    file_id,
+                    caption=text,
+                    reply_to_message_id=reply_to,
+                    parse_mode=parse_mode,
+                    reply_markup=button,
+                )
+        except MediaEmpty:
+            await self.bot.client.send_message(
+                ctx.chat.id, await self.text(ctx.chat.id, "welcome-message-expired")
+            )
+        except MessageEmpty:
+            self.log.warning("Welcome message empty on %s.", ctx.chat.id)
 
     @command.filters(filters.admin_only)
     async def cmd_goodbye(self, ctx: command.Context) -> Optional[str]:
@@ -397,7 +586,15 @@ class Greeting(plugin.Plugin):
         else:
             parse_mode = ParseMode.MARKDOWN
 
-        await ctx.respond(await self.text(chat.id, "view-goodbye", setting, clean_service))
+        view_gby = await self.text(chat.id, "view-goodbye", setting, clean_service)
+        if ctx.chat.is_forum and not await self.get_action_topic(ctx.chat):
+            view_gby += "\n\n" + await self.text(
+                chat.id,
+                "greetings-topic-default",
+                f"https://t.me/{self.bot.user.username}?start=help_topic",
+            )
+
+        await ctx.respond(view_gby)
         await ctx.respond(
             text,
             mode="reply",
